"""
Definition of all artifactory objects.
"""
import json
import logging
import os
import warnings
from os.path import isdir, join
from pathlib import Path
from typing import List, Optional, Dict, Tuple, Union, Iterator, overload

import requests
from pydantic import parse_obj_as

from pyartifactory.artifactory_object import ArtifactoryObject
from pyartifactory.exception import (
    UserNotFoundException,
    UserAlreadyExistsException,
    GroupNotFoundException,
    RepositoryAlreadyExistsException,
    GroupAlreadyExistsException,
    RepositoryNotFoundException,
    ArtifactoryException,
    InvalidTokenDataException,
    PropertyNotFoundException,
    ArtifactNotFoundException,
    BadPropertiesException,
    PermissionAlreadyExistsException,
    PermissionNotFoundException,
)
from pyartifactory.models import (
    AuthModel,
    ApiKeyModel,
    PasswordModel,
    AccessTokenModel,
    Group,
    LocalRepository,
    VirtualRepository,
    LocalRepositoryResponse,
    VirtualRepositoryResponse,
    RemoteRepository,
    RemoteRepositoryResponse,
    SimpleRepository,
    AnyRepository,
    AnyRepositoryResponse,
    UserResponse,
    NewUser,
    SimpleUser,
    User,
    ArtifactPropertiesResponse,
    ArtifactStatsResponse,
    ArtifactInfoResponse,
    Permission,
    PermissionV2,
    SimplePermission,
    AnyPermission,
)
from pyartifactory.models.artifact import (
    ArtifactFileInfoResponse,
    ArtifactFolderInfoResponse,
    ArtifactListResponse,
)
from pyartifactory.utils import custom_encoder


logger = logging.getLogger("pyartifactory")


class Artifactory:
    """Models artifactory."""

    def __init__(
        self,
        url: str,
        auth: Tuple[str, str],
        verify: bool = True,
        cert: str = None,
        api_version: int = 1,
    ):
        self.artifactory = AuthModel(
            url=url, auth=auth, verify=verify, cert=cert, api_version=api_version
        )
        self.users = ArtifactoryUser(self.artifactory)
        self.groups = ArtifactoryGroup(self.artifactory)
        self.security = ArtifactorySecurity(self.artifactory)
        self.repositories = ArtifactoryRepository(self.artifactory)
        self.artifacts = ArtifactoryArtifact(self.artifactory)
        self.permissions = ArtifactoryPermission(self.artifactory)


class ArtifactoryUser(ArtifactoryObject):
    """Models an artifactory user."""

    _uri = "security/users"

    def create(self, user: NewUser) -> UserResponse:
        """
        Create user
        :param user: NewUser object
        :return: User
        """
        username = user.name
        try:
            self.get(username)
            logger.error("User %s already exists", username)
            raise UserAlreadyExistsException(f"User {username} already exists")
        except UserNotFoundException:
            data = user.dict()
            data["password"] = user.password.get_secret_value()
            self._put(f"api/{self._uri}/{username}", json=data)
            logger.debug("User %s successfully created", username)
            return self.get(user.name)

    def get(self, name: str) -> UserResponse:
        """
        Read user from artifactory. Fill object if exist
        :param name: Name of the user to retrieve
        :return: UserModel
        """
        try:
            response = self._get(f"api/{self._uri}/{name}")
            logger.debug("User %s found", name)
            return UserResponse(**response.json())
        except requests.exceptions.HTTPError as error:
            if error.response.status_code in (404, 400):
                logger.error("User %s does not exist", name)
                raise UserNotFoundException(f"{name} does not exist")
            raise ArtifactoryException from error

    def list(self) -> List[SimpleUser]:
        """
        Lists all the users
        :return: UserList
        """
        response = self._get(f"api/{self._uri}")
        logger.debug("List all users successful")
        return [SimpleUser(**user) for user in response.json()]

    def update(self, user: User) -> UserResponse:
        """
        Updates an artifactory user
        :param user: NewUser object
        :return: UserModel
        """
        username = user.name
        self.get(username)
        self._post(
            f"api/{self._uri}/{username}",
            json=user.dict(exclude={"lastLoggedIn", "realm"}),
        )
        logger.debug("User %s successfully updated", username)
        return self.get(username)

    def delete(self, name: str) -> None:
        """
        Remove user
        :param name: Name of the user to delete
        :return: None
        """
        self.get(name)
        self._delete(f"api/{self._uri}/{name}")
        logger.debug("User %s successfully deleted", name)

    def unlock(self, name: str) -> None:
        """
        Unlock user
        Even if the user doesn't exist, it succeed too
        :param name: Name of the user to unlock
        :return none
        """
        self._post(f"api/security/unlockUsers/{name}")
        logger.debug("User % successfully unlocked", name)


class ArtifactoryPermission(ArtifactoryObject):
    """Models an artifactory permission."""

    def __init__(self, artifactory: AuthModel) -> None:
        super().__init__(artifactory)
        if self._api_version == 2:
            self._uri = "v2/security/permissions"
        if self._api_version == 1:
            self._uri = "security/permissions"

    @overload
    def create(
        self,
        permission: Permission,
    ) -> Permission:
        ...

    @overload
    def create(
        self,
        permission: PermissionV2,
    ) -> PermissionV2:
        ...

    def create(self, permission: AnyPermission) -> AnyPermission:
        """
        Creates a permission
        :param permission: Permission v2 or v1 object
        :return: Permission v2 or v1
        """
        permission_name = permission.name
        try:
            self.get(permission_name)
            logger.debug("Permission %s already exists", permission_name)
            raise PermissionAlreadyExistsException(
                f"Permission {permission_name} already exists"
            )
        except PermissionNotFoundException:
            self._put(
                f"api/{self._uri}/{permission_name}",
                json=permission.dict(by_alias=True),
            )
            logger.debug("Permission %s successfully created", permission_name)
            return self.get(permission_name)

    def get(self, permission_name: str) -> AnyPermission:
        """
        Read permission from artifactory. Fill object if exist
        :param permission_name: Name of the permission to retrieve
        :return: Permission
        """
        try:
            response = self._get(f"api/{self._uri}/{permission_name}")
            logger.debug("Permission %s found", permission_name)
            return (
                Permission(**response.json())
                if self._artifactory.api_version == 1
                else PermissionV2(**response.json())
            )
        except requests.exceptions.HTTPError as error:
            if error.response.status_code in (404, 400):
                logger.error("Permission %s does not exist", permission_name)
                raise PermissionNotFoundException(
                    f"Permission {permission_name} does not exist"
                )
            raise ArtifactoryException from error

    def list(self) -> List[SimplePermission]:
        """
        Lists all the permissions
        :return: A list of permissions
        """
        response = self._get(f"api/{self._uri}")
        logger.debug("List all permissions successful")
        return [SimplePermission(**permission) for permission in response.json()]

    @overload
    def update(self, permission: Permission) -> Permission:
        ...

    @overload
    def update(self, permission: PermissionV2) -> PermissionV2:
        ...

    def update(self, permission: AnyPermission) -> AnyPermission:
        """
        Updates an artifactory permission
        :param permission: Permission v2 or v1 object
        :return: Permission v2 or v1
        """
        permission_name = permission.name
        self._put(
            f"api/{self._uri}/{permission_name}", json=permission.dict(by_alias=True)
        )
        logger.debug("Permission %s successfully updated", permission_name)
        return self.get(permission_name)

    def delete(self, permission_name: str) -> None:
        """
        Removes an artifactory permission
        :param permission_name: Name of the permission to delete
        :return: None
        """
        self.get(permission_name)
        self._delete(f"api/{self._uri}/{permission_name}")
        logger.debug("Permission %s successfully deleted", permission_name)


class ArtifactorySecurity(ArtifactoryObject):
    """Models artifactory security."""

    _uri = "security"

    def get_encrypted_password(self) -> PasswordModel:
        """
        Get the encrypted password of the authenticated requestor.
        :return: str
        """
        response = self._get(f"api/{self._uri}/encryptedPassword")
        logger.debug("Encrypted password successfully delivered")
        return PasswordModel(**response.json())

    def create_access_token(
        self,
        user_name: str,
        expires_in: int = 3600,
        refreshable: bool = False,
        groups: Optional[List[str]] = None,
    ) -> AccessTokenModel:
        """
        Creates an access token.

        :param user_name: Name of the user to whom an access key should be granted. transient token
                          is created if user doesn't exist in artifactory.
        :param expires_in: Expiry time for the token in seconds. For eternal tokens specify 0.
        :param refreshable: If set to true token can be refreshed using the refresh token returned.
        :param groups: A list of groups the token has membership of.
                       If an existing user in artifactory is used with existing memberships
                       groups are automatically implied without specification.
        :return: AccessToken
        """
        payload = {
            "username": user_name,
            "expires_in": expires_in,
            "refreshable": refreshable,
        }
        if groups:
            if not isinstance(groups, list):
                raise ValueError(groups)
            scope = f'member-of-groups:"{",".join(groups)}"'
            payload.update({"scope": scope})
        response = self._post(
            f"api/{self._uri}/token", data=payload, raise_for_status=False
        )
        if response.ok:
            return AccessTokenModel(**response.json())
        raise InvalidTokenDataException(
            response.json().get("error_description", "Unknown error")
        )

    def revoke_access_token(self, token: str = None, token_id: str = None) -> bool:
        """
        Revokes an access token.

        :param token: The token to revoke
        :param token_id: The id of a token to revoke
        :return: bool True or False indicating success or failure of token revocation attempt.
        """
        if not any([token, token_id]):
            logger.error("Neither a token or a token id was specified")
            raise InvalidTokenDataException
        payload: Dict[str, Optional[str]] = (
            {"token": token} if token else {"token_id": token_id}
        )
        response = self._post(
            f"api/{self._uri}/token/revoke", data=payload, raise_for_status=False
        )
        if response.ok:
            logger.debug("Token revoked successfully, or token did not exist")
            return True
        logger.error("Token revocation unsuccessful, response was %s", response.text)
        return False

    def create_api_key(self) -> ApiKeyModel:
        """
        Create an API key for the current user.
        :return: Error if API key already exists - use regenerate API key instead.
        """
        response = self._post(f"api/{self._uri}/apiKey")
        logger.debug("API Key successfully created")
        return ApiKeyModel(**response.json())

    def regenerate_api_key(self) -> ApiKeyModel:
        """
        Regenerate an API key for the current user
        :return: API key
        """
        response = self._put(f"api/{self._uri}/apiKey")
        logger.debug("API Key successfully regenerated")
        return ApiKeyModel(**response.json())

    def get_api_key(self) -> ApiKeyModel:
        """
        Get the current user's own API key
        :return: API key
        """
        response = self._get(f"api/{self._uri}/apiKey")
        logger.debug("API Key successfully delivered")
        return ApiKeyModel(**response.json())

    def revoke_api_key(self) -> None:
        """
        Revokes the current user's API key
        :return: None
        """
        self._delete(f"api/{self._uri}/apiKey")
        logger.debug("API Key successfully revoked")

    def revoke_user_api_key(self, name: str) -> None:
        """
        Revokes the API key of another user
        :param name: name of the user to whom api key has to be revoked
        :return: None
        """
        self._delete(f"api/{self._uri}/apiKey/{name}")
        logger.debug("User API Key successfully revoked")


class ArtifactoryGroup(ArtifactoryObject):
    """Models artifactory groups."""

    _uri = "security/groups"

    def create(self, group: Group) -> Group:
        """
        Creates a new group in Artifactory or replaces an existing group
        :param group: Group to create
        :return: Created group
        """
        group_name = group.name
        try:
            self.get(group_name)
            logger.error("Group %s already exists", group_name)
            raise GroupAlreadyExistsException(f"Group {group_name} already exists")
        except GroupNotFoundException:
            self._put(f"api/{self._uri}/{group_name}", json=group.dict())
            logger.debug("Group %s successfully created", group_name)
            return self.get(group.name)

    def get(self, name: str) -> Group:
        """
        Get the details of an Artifactory Group
        :param name: Name of the group to retrieve
        :return: Found artifactory group
        """
        try:
            response = self._get(
                f"api/{self._uri}/{name}", params={"includeUsers": True}
            )
            logger.debug("Group %s found", name)
            return Group(**response.json())
        except requests.exceptions.HTTPError as error:
            if error.response.status_code in (404, 400):
                logger.error("Group %s does not exist", name)
                raise GroupNotFoundException(f"Group {name} does not exist")
            raise ArtifactoryException from error

    def list(self) -> List[Group]:
        """
        Lists all the groups
        :return: GroupList
        """
        response = self._get(f"api/{self._uri}")
        logger.debug("List all groups successful")
        return [Group(**group) for group in response.json()]

    def update(self, group: Group) -> Group:
        """
        Updates an exiting group in Artifactory with the provided group details.
        :param group: Group to be updated
        :return: Updated group
        """
        group_name = group.name
        self.get(group_name)
        self._post(f"api/{self._uri}/{group_name}", json=group.dict())
        logger.debug("Group %s successfully updated", group_name)
        return self.get(group_name)

    def delete(self, name: str) -> None:
        """
        Removes a group
        :param name: Name of the group to delete
        :return: None
        """
        self.get(name)
        self._delete(f"api/{self._uri}/{name}")
        logger.debug("Group %s successfully deleted", name)


class ArtifactoryRepository(ArtifactoryObject):
    """Models an artifactory repository."""

    _uri = "repositories"

    # Repositories operations
    def get_repo(self, repo_name: str) -> AnyRepositoryResponse:
        """
        Finds repository in artifactory. Raises an exception if the repo doesn't exist.
        :param repo_name: Name of the repository to retrieve
        :return: Either a local, virtual or remote repository
        """
        try:
            response = self._get(f"api/{self._uri}/{repo_name}")
            repo: AnyRepositoryResponse = parse_obj_as(
                Union[
                    LocalRepositoryResponse,
                    VirtualRepositoryResponse,
                    RemoteRepositoryResponse,
                ],
                response.json(),
            )
            return repo
        except requests.exceptions.HTTPError as error:
            if error.response.status_code in (404, 400):
                logger.error("Repository %s does not exist", repo_name)
                raise RepositoryNotFoundException(
                    f" Repository {repo_name} does not exist"
                )
            raise ArtifactoryException from error

    @overload
    def create_repo(self, repo: LocalRepository) -> LocalRepositoryResponse:
        ...

    @overload
    def create_repo(self, repo: VirtualRepository) -> VirtualRepositoryResponse:
        ...

    @overload
    def create_repo(self, repo: RemoteRepository) -> RemoteRepositoryResponse:
        ...

    def create_repo(self, repo: AnyRepository) -> AnyRepositoryResponse:
        """
        Creates a local, virtual or remote repository
        :param repo: Either a local, virtual or remote repository
        :return: LocalRepositoryResponse, VirtualRepositoryResponse or RemoteRepositoryResponse object
        """
        repo_name = repo.key
        try:
            self.get_repo(repo_name)
            logger.error("Repository %s already exists", repo_name)
            raise RepositoryAlreadyExistsException(
                f"Repository {repo_name} already exists"
            )
        except RepositoryNotFoundException:
            data = json.dumps(repo, default=custom_encoder)
            self._put(
                f"api/{self._uri}/{repo_name}",
                headers={"Content-Type": "application/json"},
                data=data,
            )
            logger.debug("Repository %s successfully created", repo_name)
            return self.get_repo(repo_name)

    @overload
    def update_repo(self, repo: LocalRepository) -> LocalRepositoryResponse:
        ...

    @overload
    def update_repo(self, repo: VirtualRepository) -> VirtualRepositoryResponse:
        ...

    @overload
    def update_repo(self, repo: RemoteRepository) -> RemoteRepositoryResponse:
        ...

    def update_repo(self, repo: AnyRepository) -> AnyRepositoryResponse:
        """
        Updates a local, virtual or remote repository
        :param repo: Either a local, virtual or remote repository
        :return: LocalRepositoryResponse, VirtualRepositoryResponse or RemoteRepositoryResponse object
        """
        repo_name = repo.key
        self.get_repo(repo_name)

        repo_dict = json.dumps(repo.dict(exclude_unset=True), default=custom_encoder)

        self._post(
            f"api/{self._uri}/{repo_name}",
            headers={"Content-Type": "application/json"},
            data=repo_dict,
        )
        logger.debug("Repository %s successfully updated", repo_name)
        return self.get_repo(repo_name)

    # Local repositories operations
    def create_local_repo(self, repo: LocalRepository) -> LocalRepositoryResponse:
        """
        Creates a new local repository
        :param repo: LocalRepository object
        :return: LocalRepositoryResponse object
        """
        warnings.warn(
            "`create_local_repo()` is deprecated, use `create_repo` instead",
            DeprecationWarning,
        )
        repo_name = repo.key
        try:
            self.get_local_repo(repo_name)
            logger.error("Repository %s already exists", repo_name)
            raise RepositoryAlreadyExistsException(
                f"Repository {repo_name} already exists"
            )
        except RepositoryNotFoundException:
            self._put(f"api/{self._uri}/{repo_name}", json=repo.dict())
            logger.debug("Repository %s successfully created", repo_name)
            return self.get_local_repo(repo_name)

    def get_local_repo(self, repo_name: str) -> LocalRepositoryResponse:
        """
        Finds repository in artifactory. Fill object if exist
        :param repo_name: Name of the repository to retrieve
        :return: LocalRepositoryResponse object
        """
        warnings.warn(
            "`get_local_repo()` is deprecated, use `get_repo` instead",
            DeprecationWarning,
        )
        try:
            response = self._get(f"api/{self._uri}/{repo_name}")
            logger.debug("Repository %s found", repo_name)
            return LocalRepositoryResponse(**response.json())
        except requests.exceptions.HTTPError as error:
            if error.response.status_code in (404, 400):
                raise RepositoryNotFoundException(
                    f" Repository {repo_name} does not exist"
                )
            raise ArtifactoryException from error

    def update_local_repo(self, repo: LocalRepository) -> LocalRepositoryResponse:
        """
        Updates an artifactory repository
        :param repo: LocalRepository object
        :return: LocalRepositoryResponse
        """
        warnings.warn(
            "`update_local_repo()` is deprecated, use `update_repo` instead",
            DeprecationWarning,
        )
        repo_name = repo.key
        self.get_local_repo(repo_name)
        self._post(f"api/{self._uri}/{repo_name}", json=repo.dict())
        logger.debug("Repository %s successfully updated", repo_name)
        return self.get_local_repo(repo_name)

    # Virtual repositories operations
    def create_virtual_repo(self, repo: VirtualRepository) -> VirtualRepositoryResponse:
        """
        Creates a new local repository
        :param repo: VirtualRepository object
        :return: VirtualRepositoryResponse object
        """
        warnings.warn(
            "`create_virtual_repo()` is deprecated, use `create_repo` instead",
            DeprecationWarning,
        )
        repo_name = repo.key
        try:
            self.get_virtual_repo(repo_name)
            logger.error("Repository %s already exists", repo_name)
            raise RepositoryAlreadyExistsException(
                f"Repository {repo_name} already exists"
            )
        except RepositoryNotFoundException:
            self._put(f"api/{self._uri}/{repo_name}", json=repo.dict())
            logger.debug("Repository %s successfully created", repo_name)
            return self.get_virtual_repo(repo_name)

    def get_virtual_repo(self, repo_name: str) -> VirtualRepositoryResponse:
        """
        Finds repository in artifactory. Fill object if exist
        :param repo_name: Name of the repository to retrieve
        :return: VirtualRepositoryResponse object
        """
        warnings.warn(
            "`get_virtual_repo()` is deprecated, use `get_repo` instead",
            DeprecationWarning,
        )
        try:
            response = self._get(f"api/{self._uri}/{repo_name}")
            logger.debug("Repository %s found", repo_name)
            return VirtualRepositoryResponse(**response.json())
        except requests.exceptions.HTTPError as error:
            if error.response.status_code in (404, 400):
                raise RepositoryNotFoundException(
                    f" Repository {repo_name} does not exist"
                )
            raise ArtifactoryException from error

    def update_virtual_repo(self, repo: VirtualRepository) -> VirtualRepositoryResponse:
        """
        Updates a virtual artifactory repository
        :param repo: VirtualRepository object
        :return: VirtualRepositoryResponse
        """
        warnings.warn(
            "`update_virtual_repo()` is deprecated, use `update_repo` instead",
            DeprecationWarning,
        )
        repo_name = repo.key
        self.get_virtual_repo(repo_name)
        self._post(f"api/{self._uri}/{repo_name}", json=repo.dict())
        logger.debug("Repository %s successfully updated", repo_name)
        return self.get_virtual_repo(repo_name)

    # Remote repositories operations
    def create_remote_repo(self, repo: RemoteRepository) -> RemoteRepositoryResponse:
        """
        Creates a new local repository
        :param repo: RemoteRepository object
        :return: RemoteRepositoryResponse object
        """
        warnings.warn(
            "`create_remote_repo()` is deprecated, use `create_repo` instead",
            DeprecationWarning,
        )
        repo_name = repo.key
        try:
            self.get_remote_repo(repo_name)
            logger.error("Repository %s already exists", repo_name)
            raise RepositoryAlreadyExistsException(
                f"Repository {repo_name} already exists"
            )
        except RepositoryNotFoundException:
            self._put(f"api/{self._uri}/{repo_name}", json=repo.dict())
            logger.debug("Repository %s successfully created", repo_name)
            return self.get_remote_repo(repo_name)

    def get_remote_repo(self, repo_name: str) -> RemoteRepositoryResponse:
        """
        Finds a remote repository in artifactory. Fill object if exist
        :param repo_name: Name of the repository to retrieve
        :return: RemoteRepositoryResponse object
        """
        warnings.warn(
            "`get_remote_repo()` is deprecated, use `get_repo` instead",
            DeprecationWarning,
        )
        try:
            response = self._get(f"api/{self._uri}/{repo_name}")
            logger.debug("Repository %s found", repo_name)
            return RemoteRepositoryResponse(**response.json())
        except requests.exceptions.HTTPError as error:
            if error.response.status_code in (404, 400):
                raise RepositoryNotFoundException(
                    f" Repository {repo_name} does not exist"
                )
            raise ArtifactoryException from error

    def update_remote_repo(self, repo: RemoteRepository) -> RemoteRepositoryResponse:
        """
        Updates a remote artifactory repository
        :param repo: VirtualRepository object
        :return: VirtualRepositoryResponse
        """
        warnings.warn(
            "`update_remote_repo()` is deprecated, use `update_repo` instead",
            DeprecationWarning,
        )
        repo_name = repo.key
        self.get_remote_repo(repo_name)
        self._post(f"api/{self._uri}/{repo_name}", json=repo.dict())
        logger.debug("Repository %s successfully updated", repo_name)
        return self.get_remote_repo(repo_name)

    def list(self) -> List[SimpleRepository]:
        """
        Lists all the repositories
        :return: A list of repositories
        """
        response = self._get(f"api/{self._uri}")
        logger.debug("List all repositories successful")
        return [SimpleRepository(**repository) for repository in response.json()]

    def delete(self, repo_name: str) -> None:
        """
        Removes a local repository
        :param repo_name: Name of the repository to delete
        :return: None
        """

        self._delete(f"api/{self._uri}/{repo_name}")
        logger.debug("Repository %s successfully deleted", repo_name)


class ArtifactoryArtifact(ArtifactoryObject):
    """Models an artifactory artifact."""

    def _walk(
        self, artifact_path: str, topdown: bool = True
    ) -> Iterator[ArtifactInfoResponse]:
        """Iterate over artifact (file or directory) recursively.

        :param artifact_path: Path to file or folder in Artifactory
        :param topdown: True for a top-down (directory first) traversal
        """
        info = self.info(artifact_path)
        if not isinstance(info, ArtifactFolderInfoResponse):
            yield info
        else:
            if topdown:
                yield info
            for subdir in (child for child in info.children if child.folder is True):
                yield from self._walk(artifact_path + subdir.uri, topdown=topdown)
            for file in (child for child in info.children if child.folder is not True):
                yield from self._walk(artifact_path + file.uri, topdown=topdown)
            if not topdown:
                yield info

    def info(self, artifact_path: str) -> ArtifactInfoResponse:
        """
        Retrieve information about a file or a folder

        See https://www.jfrog.com/confluence/display/JFROG/Artifactory+REST+API#ArtifactoryRESTAPI-FolderInfo
        and https://www.jfrog.com/confluence/display/JFROG/Artifactory+REST+API#ArtifactoryRESTAPI-FileInfo

        :param artifact_path: Path to file or folder in Artifactory
        """
        artifact_path = artifact_path.lstrip("/")
        try:
            response = self._get(f"api/storage/{artifact_path}")
            artifact_info: ArtifactInfoResponse = parse_obj_as(
                Union[ArtifactFolderInfoResponse, ArtifactFileInfoResponse],
                response.json(),
            )
            return artifact_info
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 404:
                logger.error("Artifact %s does not exist", artifact_path)
                raise ArtifactNotFoundException(
                    f"Artifact {artifact_path} does not exist"
                )
            raise ArtifactoryException from error

    def deploy(
        self, local_file_location: str, artifact_path: str
    ) -> ArtifactInfoResponse:
        """
        Deploy a file or directory.
        :param artifact_path: Path to artifactory in Artifactory
        :param local_file_location: Location of the file or folder to deploy
        """
        if isdir(local_file_location):
            for root, _, files in os.walk(local_file_location):
                new_root = f"{artifact_path}/{root[len(local_file_location):]}"
                for file in files:
                    self.deploy(join(root, file), f"{new_root}/{file}")
        else:
            artifact_path = artifact_path.lstrip("/")
            local_filename = artifact_path.split("/")[-1]
            # we need to silence a bogus mypy error as `file` is already used above
            # with a different type. See https://github.com/python/mypy/issues/6232
            with open(local_file_location, "rb") as file:  # type: ignore[assignment]
                self._put(f"{artifact_path}", data=file)
                logger.debug("Artifact %s successfully deployed", local_filename)
        return self.info(artifact_path)

    def _download(self, artifact_path: str, local_directory_path: Path = None) -> Path:
        """
        Download artifact (file) into local directory.
        :param artifact_path: Path to file in Artifactory
        :param local_directory_path: Local path to where the artifact will be downloaded
        :return: File name
        """
        artifact_path = artifact_path.lstrip("/")
        local_filename = artifact_path.split("/")[-1]

        if local_directory_path:
            local_directory_path.mkdir(parents=True, exist_ok=True)
            local_file_full_path = local_directory_path / local_filename
        else:
            local_file_full_path = Path(local_filename)

        with self._get(f"{artifact_path}", stream=True) as response:
            with open(local_file_full_path, "wb") as file:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:  # filter out keep-alive new chunks
                        file.write(chunk)
        logger.debug("Artifact %s successfully downloaded", local_filename)
        return local_file_full_path

    @staticmethod
    def _get_path_prefix(artifact_path: str):
        if artifact_path.startswith("/"):
            artifact_path = artifact_path[1:]
        return artifact_path.rsplit("/", 1)[0] + "/" if "/" in artifact_path else ""

    @staticmethod
    def _remove_prefix(_str: str, prefix: str) -> str:
        if _str.startswith(prefix):
            return _str[len(prefix) :]
        raise ValueError(f"Input string, '{_str}', doesn't have the prefix: '{prefix}'")

    def download(self, artifact_path: str, local_directory_path: str = ".") -> str:
        """
        Download artifact (file or directory) into local directory.
        :param artifact_path: Path to file or directory in Artifactory
        :param local_directory_path: Local path to where the artifact will be downloaded
        :return: File name
        """
        artifact_path = artifact_path.rstrip("/")
        basename = artifact_path.split("/")[-1]
        prefix = self._get_path_prefix(artifact_path)
        for art in self._walk(artifact_path):
            full_path = art.repo + art.path
            local_artifact_path = self._remove_prefix(full_path, prefix)
            local_path = Path(local_directory_path) / local_artifact_path
            if isinstance(art, ArtifactFolderInfoResponse):
                os.makedirs(local_path, exist_ok=True)
            else:
                self._download(full_path, local_path.parent)
        return f"{local_directory_path}/{basename}"

    def list(
        self,
        artifact_path: str,
        recursive: bool = True,
        depth: Optional[int] = None,
        list_folders: bool = True,
    ) -> ArtifactListResponse:
        """
        Retrieve a list of files or a folders

        See https://www.jfrog.com/confluence/display/JFROG/Artifactory+REST+API#ArtifactoryRESTAPI-FileList

        :param artifact_path: Path to folder in Artifactory
        :param recursive: Recursively retrieve files and folders
        :param depth: The depth to recursively retrieve
        :param list_folders: Whether or not to include folders in the response
        :return: A list of artifacts in Artifactory
        """
        try:
            params = {
                "deep": int(recursive),
                "listFolders": int(list_folders),
            }
            if depth is not None:
                params.update(depth=depth)
            response = self._get(f"api/storage/{artifact_path}?list", params=params)
            artifact_list: ArtifactListResponse = parse_obj_as(
                ArtifactListResponse, response.json()
            )
            return artifact_list
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 404:
                logger.error("Artifact %s does not exist", artifact_path)
                raise ArtifactNotFoundException(
                    f"Artifact {artifact_path} does not exist"
                )
            raise ArtifactoryException from error

    def properties(
        self, artifact_path: str, properties: Optional[List[str]] = None
    ) -> ArtifactPropertiesResponse:
        """
        :param artifact_path: Path to file in Artifactory
        :param properties: List of properties to retrieve
        :return: Artifact properties
        """
        if properties is None:
            properties = []
        artifact_path = artifact_path.lstrip("/")
        try:
            response = self._get(
                f"api/storage/{artifact_path}",
                params={"properties": ",".join(properties)},
            )
            logger.debug("Artifact Properties successfully retrieved")
            return ArtifactPropertiesResponse(**response.json())
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 404:
                raise PropertyNotFoundException(
                    f"Properties {properties} were not found on artifact {artifact_path}"
                )
            raise ArtifactoryException from error

    def set_properties(
        self,
        artifact_path: str,
        properties: Dict[str, List[str]],
        recursive: bool = True,
    ) -> ArtifactPropertiesResponse:
        """
        :param artifact_path: Path to file or folder in Artifactory
        :param properties: List of properties to update
        :param recursive: If set to true, properties will be applied recursively to subfolders and files
        :return: None
        """
        if properties is None:
            properties = {}
        artifact_path = artifact_path.lstrip("/")
        properties_param_str = ""
        for k, v in properties.items():
            values_str = ",".join(v)
            properties_param_str += f"{k}={values_str};"
        try:
            self._put(
                f"api/storage/{artifact_path}",
                params={
                    "recursive": int(recursive),
                    "properties": properties_param_str,
                },
            )
            logger.debug("Artifact Properties successfully set")
            return self.properties(artifact_path)
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 404:
                logger.error("Artifact %s does not exist", artifact_path)
                raise ArtifactNotFoundException(
                    f"Artifact {artifact_path} does not exist"
                )
            if error.response.status_code == 400:
                logger.error("A property value includes forbidden special characters")
                raise BadPropertiesException(
                    "A property value includes forbidden special characters"
                )
            raise ArtifactoryException from error

    def update_properties(
        self,
        artifact_path: str,
        properties: Dict[str, List[str]],
        recursive: bool = True,
    ) -> ArtifactPropertiesResponse:
        """
        :param artifact_path: Path to file or folder in Artifactory
        :param properties: List of properties to update
        :param recursive: If set to true, properties will be applied recursively to subfolders and files
        :return: None
        """
        if properties is None:
            properties = {}
        artifact_path = artifact_path.lstrip("/")
        try:
            self._patch(
                f"api/metadata/{artifact_path}",
                params={"recursive": int(recursive)},
                headers={"Content-Type": "application/json"},
                json={"props": properties},
            )
            logger.debug("Artifact Properties successfully updated")
            return self.properties(artifact_path)
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 400:
                logger.error("Error updating artifact properties")
                raise ArtifactoryException("Error updating artifact properties")
            raise ArtifactoryException from error

    def stats(self, artifact_path: str) -> ArtifactStatsResponse:
        """
        :param artifact_path: Path to file in Artifactory
        :return: Artifact Stats
        """
        artifact_path = artifact_path.lstrip("/")
        response = self._get(f"api/storage/{artifact_path}?stats")
        logger.debug("Artifact stats successfully retrieved")
        return ArtifactStatsResponse(**response.json())

    def copy(
        self, artifact_current_path: str, artifact_new_path: str, dryrun: bool = False
    ) -> ArtifactInfoResponse:
        """
        :param artifact_current_path: Current path to file
        :param artifact_new_path: New path to file
        :param dryrun: Dry run
        :return: ArtifactInfoResponse: info of the copied artifact
        """
        artifact_current_path = artifact_current_path.lstrip("/")
        artifact_new_path = artifact_new_path.lstrip("/")
        if dryrun:
            dry = 1
        else:
            dry = 0

        self._post(f"api/copy/{artifact_current_path}?to={artifact_new_path}&dry={dry}")
        logger.debug("Artifact %s successfully copied", artifact_current_path)
        return self.info(artifact_new_path)

    def move(
        self, artifact_current_path: str, artifact_new_path: str, dryrun: bool = False
    ) -> ArtifactInfoResponse:
        """
        :param artifact_current_path: Current path to file
        :param artifact_new_path: New path to file
        :param dryrun: Dry run
        :return: ArtifactInfoResponse: info of the moved artifact
        """
        artifact_current_path = artifact_current_path.lstrip("/")
        artifact_new_path = artifact_new_path.lstrip("/")

        if dryrun:
            dry = 1
        else:
            dry = 0

        self._post(f"api/move/{artifact_current_path}?to={artifact_new_path}&dry={dry}")
        logger.debug("Artifact %s successfully moved", artifact_current_path)
        return self.info(artifact_new_path)

    def delete(self, artifact_path: str) -> None:
        """
        :param artifact_path: Path to file in Artifactory
        :return: bool
        """
        artifact_path = artifact_path.lstrip("/")
        self._delete(f"{artifact_path}")
        logger.debug("Artifact %s successfully deleted", artifact_path)



"""
Definition of all exceptions.
"""


class ArtifactoryException(Exception):
    """Generic artifactory exception."""


class UserAlreadyExistsException(ArtifactoryException):
    """User already exists."""


class GroupAlreadyExistsException(ArtifactoryException):
    """Group already exists."""


class RepositoryAlreadyExistsException(ArtifactoryException):
    """Repository already exists."""


class PermissionAlreadyExistsException(ArtifactoryException):
    """Permission already exists."""


class UserNotFoundException(ArtifactoryException):
    """The user was not found."""


class GroupNotFoundException(ArtifactoryException):
    """The group was not found."""


class RepositoryNotFoundException(ArtifactoryException):
    """The repository was not found."""


class PermissionNotFoundException(ArtifactoryException):
    """A permission object was not found."""


class ArtifactNotFoundException(ArtifactoryException):
    """An artifact was not found"""


class BadPropertiesException(ArtifactoryException):
    """Property value includes invalid characters"""


class PropertyNotFoundException(ArtifactoryException):
    """All requested properties were not found"""


class InvalidTokenDataException(ArtifactoryException):
    """The token contains invalid data."""



"""
Definition of all artifact models.
"""


from datetime import datetime
from typing import Optional, List, Dict, Union

try:
    from typing import Literal  # type: ignore
except ImportError:
    from typing_extensions import Literal

from pydantic import BaseModel


class Checksums(BaseModel):
    """Models a checksum."""

    sha1: str
    md5: str
    sha256: str


class OriginalChecksums(BaseModel):
    """Models original checksums."""

    sha256: str


class Child(BaseModel):
    """Models a child folder."""

    uri: str
    folder: bool


class ArtifactPropertiesResponse(BaseModel):
    """Models an artifact properties response."""

    uri: str
    properties: Dict[str, List[str]]


class ArtifactInfoResponseBase(BaseModel):
    """The base information available for both file and folder"""

    repo: str
    path: str
    created: Optional[datetime] = None
    createdBy: Optional[str] = None
    lastModified: Optional[datetime] = None
    modifiedBy: Optional[str] = None
    lastUpdated: Optional[datetime] = None
    uri: str


class ArtifactFolderInfoResponse(ArtifactInfoResponseBase):
    """Models an artifact folder info response."""

    children: List[Child]


class ArtifactFileInfoResponse(ArtifactInfoResponseBase):
    """Models an artifact file info response."""

    downloadUri: Optional[str] = None
    remoteUrl: Optional[str] = None
    mimeType: Optional[str] = None
    size: Optional[int] = None
    checksums: Optional[Checksums] = None
    originalChecksums: Optional[OriginalChecksums] = None


class ArtifactListEntryResponse(BaseModel):
    """Base model for an entry in an artifact list response."""

    uri: str
    size: int
    lastModified: datetime
    folder: bool


class ArtifactListFolderResponse(ArtifactListEntryResponse):
    """Models a folder in an artifact list response."""

    folder: Literal[True]


class ArtifactListFileResponse(ArtifactListEntryResponse):
    """Models a file in an artifact list response."""

    folder: Literal[False]
    sha1: Optional[str] = None
    sha2: Optional[str] = None


class ArtifactListResponse(BaseModel):
    """Models an artifact list response."""

    uri: str
    created: datetime
    files: List[Union[ArtifactListFileResponse, ArtifactListFolderResponse]]


class ArtifactStatsResponse(BaseModel):
    """Models an artifact statistics response."""

    uri: str
    downloadCount: int
    lastDownloaded: int
    lastDownloadedBy: Optional[str]
    remoteDownloadCount: int
    remoteLastDownloaded: int


ArtifactInfoResponse = Union[ArtifactFileInfoResponse, ArtifactFolderInfoResponse]



"""
Import all object definitions here.
"""
from pyartifactory.objects import (
    ArtifactoryUser,
    ArtifactoryGroup,
    ArtifactorySecurity,
    ArtifactoryRepository,
    ArtifactoryArtifact,
    ArtifactoryPermission,
    Artifactory,
    AccessTokenModel,
)


__version__ = "1.11.0"



"""
Definition of all permission models.
"""

from enum import Enum
from typing import List, Dict, Optional

from pydantic import BaseModel, Field


class SimplePermission(BaseModel):
    """Models a simple permission."""

    name: str
    uri: str


class PermissionEnum(str, Enum):
    """Enumerates a permission."""

    admin = "m"
    delete = "d"
    deploy = "w"
    annotate = "n"
    read = "r"


class PrincipalsPermission(BaseModel):
    """Models a principals permission."""

    users: Optional[Dict[str, List[PermissionEnum]]] = None
    groups: Optional[Dict[str, List[PermissionEnum]]] = None


class Permission(BaseModel):
    """Models a permission."""

    name: str
    includesPattern: str = "**"
    excludesPattern: str = ""
    repositories: List[str]
    principals: PrincipalsPermission


class PermissionEnumV2(str, Enum):
    """Enumerates a permission."""

    manage = "manage"
    delete = "delete"
    write = "write"
    annotate = "annotate"
    read = "read"
    distribute = "distribute"
    managedXrayMeta = "managedXrayMeta"


class PrincipalsPermissionV2(BaseModel):
    """Models a principals permission API v2."""

    users: Optional[Dict[str, List[PermissionEnumV2]]] = None
    groups: Optional[Dict[str, List[PermissionEnumV2]]] = None


class BasePermissionV2(BaseModel):
    """Models the base of a permission v2 API."""

    repositories: List[str]
    actions: PrincipalsPermissionV2
    # Note: the Jfrog API changed these parameters names in v2,
    # from 'includesPattern' to 'include-patterns'. Because they are not
    # valid Python identifiers we chose a camelCase name for the variable,
    # and specify the alias Pydantic should look for when deserializing.
    # Note that when serializing this model Pydantic will by default use the
    # identifier name, *not* the alias, so you need to pass the parameter
    # by_alias=True when exporting (like permission.json(by_alias=True))
    includePatterns: List[str] = Field(["**"], alias="include-patterns")
    excludePatterns: List[str] = Field([""], alias="exclude-patterns")

    class Config:
        """We need this to be able to use 'includePatterns' in the constructor"""

        allow_population_by_field_name = True


class RepoV2(BasePermissionV2):
    """Models a repo v2 API."""


class BuildV2(BasePermissionV2):
    """Models a build v2 API."""

    includePatterns: List[str] = Field([""], alias="include-patterns")
    repositories: List[str] = ["artifactory-build-info"]


class ReleaseBundleV2(BasePermissionV2):
    """Models a releaseBundle v2 API."""

    excludePatterns: List[str] = Field([], alias="exclude-patterns")


class PermissionV2(BaseModel):
    """Models a permission v2 API."""

    name: str
    repo: Optional[RepoV2] = None
    build: Optional[BuildV2] = None
    releaseBundle: Optional[ReleaseBundleV2] = None



"""
Import all models here.
"""
from typing import Union, Type

from .auth import AuthModel, ApiKeyModel, PasswordModel, AccessTokenModel
from .group import Group, SimpleGroup
from .user import NewUser, UserResponse, BaseUserModel, SimpleUser, User
from .repository import (
    LocalRepository,
    LocalRepositoryResponse,
    VirtualRepository,
    VirtualRepositoryResponse,
    RemoteRepository,
    RemoteRepositoryResponse,
    SimpleRepository,
)

from .artifact import (
    ArtifactPropertiesResponse,
    ArtifactStatsResponse,
    ArtifactFileInfoResponse,
    ArtifactFolderInfoResponse,
    ArtifactInfoResponse,
)
from .permission import Permission, PermissionV2, SimplePermission


AnyRepositoryResponse = Union[
    LocalRepositoryResponse, VirtualRepositoryResponse, RemoteRepositoryResponse
]

AnyRepository = Union[LocalRepository, VirtualRepository, RemoteRepository]
AnyPermission = Union[Permission, PermissionV2]



"""
Definition of all utils.
"""

from typing import Any

from pydantic import SecretStr
from pydantic.json import pydantic_encoder


def custom_encoder(obj: Any) -> Any:
    """
    Custom encoder function to be passed to the default argument of json.dumps()
    :param obj: A pydantic object
    :return: An encoded pydantic object
    """
    if isinstance(obj, SecretStr):
        return obj.get_secret_value()
    return pydantic_encoder(obj)



"""
Definition of all repository models.
"""
from enum import Enum
from typing import Optional, List
from typing_extensions import Literal

from pydantic import BaseModel, SecretStr


class PackageTypeEnum(str, Enum):
    """Enumerates package types."""

    maven = "maven"
    gradle = "gradle"
    ivy = "ivy"
    sbt = "sbt"
    helm = "helm"
    cocoapods = "cocoapods"
    opkg = "opkg"
    rpm = "rpm"
    nuget = "nuget"
    cran = "cran"
    gems = "gems"
    npm = "npm"
    bower = "bower"
    debian = "debian"
    pypi = "pypi"
    docker = "docker"
    yum = "yum"
    vcs = "vcs"
    composer = "composer"
    go = "go"
    p2 = "p2"
    chef = "chef"
    puppet = "puppet"
    generic = "generic"
    conan = "conan"
    alpine = "alpine"
    gitlfs = "gitlfs"
    vagrant = "vagrant"
    conda = "conda"
    cargo = "cargo"


class RClassEnum(str, Enum):
    """Enumerates remote types."""

    local = "local"
    virtual = "virtual"
    remote = "remote"


class ChecksumPolicyType(str, Enum):
    """Enumerates checksum policy types."""

    client_checksums = "client-checksums"
    server_generated_checksums = "server-generated-checksums"


class SnapshotVersionBehavior(str, Enum):
    """Enumerates snapshot version behavior options."""

    unique = "unique"
    non_unique = "non-unique"
    deployer = "deployer"


class PomRepoRefCleanupPolicy(str, Enum):
    """Models a repo reference cleanup policy."""

    discard_active_reference = "discard_active_reference"
    discard_any_reference = "discard_any_reference"
    nothing = "nothing"


class VcsGitProviderEnum(str, Enum):
    """Enumerates the available vcs providers."""

    github = "GITHUB"
    bitbucket = "BITBUCKET"
    oldstash = "OLDSTASH"
    stash = "STASH"
    artifactory = "ARTIFACTORY"
    custom = "CUSTOM"


class Statistics(BaseModel):
    """Models statistics."""

    enabled: bool = False


class Properties(BaseModel):
    """Models properties."""

    enabled: bool = False


class Source(BaseModel):
    """Models a source."""

    originAbsenceDetection: bool = False


class ContentSynchronisation(BaseModel):
    """Models a content synchronization."""

    enabled: bool = False
    statistics: Statistics = Statistics()
    properties: Properties = Properties()
    source: Source = Source()


class SimpleRepository(BaseModel):
    """Models a simple repository."""

    key: str
    type: str
    description: Optional[str] = None
    url: str
    packageType: str


class BaseRepositoryModel(BaseModel):
    """Models a base repository."""

    key: str
    projectKey: Optional[str] = None
    rclass: RClassEnum
    packageType: PackageTypeEnum = PackageTypeEnum.generic
    description: Optional[str] = None
    notes: Optional[str] = None
    includesPattern: str = "**/*"
    excludesPattern: str = ""
    repoLayoutRef: str = "maven-2-default"


class LocalRepository(BaseRepositoryModel):
    """Models a local repository."""

    rclass: Literal[RClassEnum.local] = RClassEnum.local
    checksumPolicyType: ChecksumPolicyType = ChecksumPolicyType.client_checksums
    handleReleases: bool = True
    handleSnapshots: bool = True
    maxUniqueSnapshots: int = 0
    maxUniqueTags: int = 0
    debianTrivialLayout: bool = False
    snapshotVersionBehavior: SnapshotVersionBehavior = (
        SnapshotVersionBehavior.non_unique
    )
    suppressPomConsistencyChecks: bool = False
    blackedOut: bool = False
    xrayIndex: bool = False
    propertySets: Optional[List[str]] = None
    dockerApiVersion: str = "V2"
    archiveBrowsingEnabled: bool = False
    calculateYumMetadata: bool = False
    yumRootDepth: int = 0
    enableFileListsIndexing: str = "false"
    optionalIndexCompressionFormats: Optional[List[str]] = None
    downloadRedirect: str = "false"
    cdnRedirect: str = "false"
    blockPushingSchema1: str = "false"
    primaryKeyPairRef: Optional[str] = None
    secondaryKeyPairRef: Optional[str] = None
    priorityResolution: str = "false"


class LocalRepositoryResponse(LocalRepository):
    """Models a local repository response."""

    enableComposerSupport: bool = False
    enableNuGetSupport: bool = False
    enableGemsSupport: bool = False
    enableNpmSupport: bool = False
    enableBowerSupport: bool = False
    enableCocoaPodsSupport: bool = False
    enableConanSupport: bool = False
    enableDebianSupport: bool = False
    enablePypiSupport: bool = False
    enablePuppetSupport: bool = False
    enableDockerSupport: bool = False
    forceNugetAuthentication: bool = False
    enableVagrantSupport: bool = False
    enableGitLfsSupport: bool = False
    enableDistRepoSupport: bool = False


class VirtualRepository(BaseRepositoryModel):
    """Models a virtual repository."""

    rclass: Literal[RClassEnum.virtual] = RClassEnum.virtual
    repositories: Optional[List[str]] = None
    artifactoryRequestsCanRetrieveRemoteArtifacts: bool = False
    debianTrivialLayout: bool = False
    keyPair: Optional[str] = None
    pomRepositoryReferencesCleanupPolicy: PomRepoRefCleanupPolicy = (
        PomRepoRefCleanupPolicy.discard_active_reference
    )
    defaultDeploymentRepo: Optional[str] = None
    forceMavenAuthentication: bool = False
    externalDependenciesEnabled: bool = False
    externalDependenciesPatterns: Optional[List[str]] = None
    externalDependenciesRemoteRepo: Optional[str] = None
    primaryKeyPairRef: Optional[str] = None
    secondaryKeyPairRef: Optional[str] = None


class VirtualRepositoryResponse(VirtualRepository):
    """Models a virtual repository response."""

    dockerApiVersion: str = "V2"
    enableComposerSupport: bool = False
    enableNuGetSupport: bool = False
    enableGemsSupport: bool = False
    enableNpmSupport: bool = False
    enableBowerSupport: bool = False
    enableCocoaPodsSupport: bool = False
    enableConanSupport: bool = False
    enableDebianSupport: bool = False
    enablePypiSupport: bool = False
    enablePuppetSupport: bool = False
    enableDockerSupport: bool = False
    forceNugetAuthentication: bool = False
    enableVagrantSupport: bool = False
    enableGitLfsSupport: bool = False
    enableDistRepoSupport: bool = False


class RemoteRepository(BaseRepositoryModel):
    """Models a remote Repository."""

    rclass: Literal[RClassEnum.remote] = RClassEnum.remote
    url: str
    username: Optional[str] = None
    password: Optional[SecretStr] = None
    proxy: Optional[str] = None
    remoteRepoChecksumPolicyType: str = "generate-if-absent"
    handleReleases: bool = True
    handleSnapshots: bool = True
    maxUniqueSnapshots: int = 0
    suppressPomConsistencyChecks: bool = False
    hardFail: bool = False
    offline: bool = False
    blackedOut: bool = False
    storeArtifactsLocally: bool = True
    socketTimeoutMillis: int = 15000
    localAddress: Optional[str] = None
    retrievalCachePeriodSecs: int = 43200
    failedRetrievalCachePeriodSecs: int = 30
    missedRetrievalCachePeriodSecs: int = 7200
    metadataRetrievalTimeoutSecs: int = 60
    unusedArtifactsCleanupEnabled: bool = False
    unusedArtifactsCleanupPeriodHours: int = 0
    assumedOfflinePeriodSecs: int = 300
    fetchJarsEagerly: int = False
    fetchSourcesEagerly: int = False
    shareConfiguration: bool = False
    synchronizeProperties: bool = False
    blockMismatchingMimeTypes: bool = True
    propertySets: Optional[List[str]] = None
    allowAnyHostAuth: bool = False
    enableCookieManagement: bool = False
    enableTokenAuthentication: bool = False
    bowerRegistryUrl: str = "https://registry.bower.io"
    gitRegistryUrl: str = "https://github.com/rust-lang/crates.io-index"
    composerRegistryUrl: str = "https://packagist.org"
    pyPIRegistryUrl: str = "https://pypi.org"
    vcsType: str = "GIT"
    vcsGitProvider: VcsGitProviderEnum = VcsGitProviderEnum.github
    vcsGitDownloadUrl: str = ""
    bypassHeadRequests: bool = False
    clientTlsCertificate: str = ""
    externalDependenciesEnabled: bool = False
    externalDependenciesPatterns: List[str] = ["**/*microsoft*/**", "**/*github*/**"]
    downloadRedirect: bool = False
    cdnRedirect: str = "false"
    contentSynchronisation: ContentSynchronisation = ContentSynchronisation()
    feedContextPath: str = "api/v2"
    downloadContextPath: str = "api/v2/package"
    v3FeedUrl: str = "https://api.nuget.org/v3/index.json"
    blockPushingSchema1: bool = False
    priorityResolution: bool = False
    disableUrlNormalization: bool = False
    xrayIndex: bool = False


class RemoteRepositoryResponse(RemoteRepository):
    """Models a RemoteRepositoryResponse."""

    dockerApiVersion: str = "V2"
    debianTrivialLayout: bool = False
    enableComposerSupport: bool = False
    enableNuGetSupport: bool = False
    enableGemsSupport: bool = False
    enableNpmSupport: bool = False
    enableBowerSupport: bool = False
    enableCocoaPodsSupport: bool = False
    enableConanSupport: bool = False
    enableDebianSupport: bool = False
    enablePypiSupport: bool = False
    enablePuppetSupport: bool = False
    enableDockerSupport: bool = False
    forceNugetAuthentication: bool = False
    enableVagrantSupport: bool = False
    enableGitLfsSupport: bool = False
    enableDistRepoSupport: bool = False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyartifactory', 'pyartifactory.models']

package_data = \
{'': ['*']}

install_requires = \
['email_validator>=1.0,<2.0', 'pydantic>=1.4,<2.0', 'requests>=2.21,<3.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['typing_extensions>=3.7.4,<4.0.0'],
 ':python_version >= "3.7" and python_version < "4.0"': ['typing_extensions>=4,<5']}

setup_kwargs = {
    'name': 'pyartifactory',
    'version': '1.11.0',
    'description': 'Typed interactions with the Jfrog Artifactory REST API',
    'long_description': '# PyArtifactory\n\n[![GitHub Actions workflow](https://github.com/anancarv/python-artifactory/workflows/Check%20code/badge.svg)](https://github.com/anancarv/python-artifactory/actions)\n[![PyPI version](https://badge.fury.io/py/pyartifactory.svg)](https://badge.fury.io/py/pyartifactory)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/c02851e5b9f24fe299783b48eab18f54)](https://www.codacy.com/gh/anancarv/python-artifactory/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=anancarv/python-artifactory&amp;utm_campaign=Badge_Grade)\n[![Codacy Badge](https://app.codacy.com/project/badge/Coverage/c02851e5b9f24fe299783b48eab18f54)](https://www.codacy.com/gh/anancarv/python-artifactory/dashboard?utm_source=github.com&utm_medium=referral&utm_content=anancarv/python-artifactory&utm_campaign=Badge_Coverage)\n![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)\n\n\n`pyartifactory` is a Python library to access the [Artifactory REST API](https://www.jfrog.com/confluence/display/RTF/Artifactory+REST+API).\n\nThis library enables you to manage Artifactory resources such as users, groups, permissions, repositories, artifacts and access tokens in your applications. Based on Python 3.6+ type hints.\n\n<!-- toc -->\n\n- [Requirements](#requirements)\n- [Install](#install)\n- [Usage](#usage)\n  * [Authentication](#authentication)\n  * [SSL Cert Verification Options](#ssl-cert-verification-options)\n  * [Admin objects](#admin-objects)\n    + [User](#user)\n    + [Group](#group)\n    + [Security](#security)\n    + [Repository](#repository)\n    + [Permission](#permission)\n      - [Artifactory lower than 6.6.0](#artifactory-lower-than-660)\n      - [Artifactory 6.6.0 or higher](#artifactory-660-or-higher)\n  * [Artifacts](#artifacts)\n    + [Get the information about a file or folder](#get-the-information-about-a-file-or-folder)\n    + [Deploy an artifact](#deploy-an-artifact)\n    + [Download an artifact](#download-an-artifact)\n    + [Retrieve artifact list](#retrieve-artifact-list)\n    + [Retrieve artifact properties](#retrieve-artifact-properties)\n    + [Set artifact properties](#set-artifact-properties)\n    + [Update artifact properties](#update-artifact-properties)\n    + [Retrieve artifact stats](#retrieve-artifact-stats)\n    + [Copy artifact to a new location](#copy-artifact-to-a-new-location)\n    + [Move artifact to a new location](#move-artifact-to-a-new-location)\n    + [Delete an artifact](#delete-an-artifact)\n  * [Contributing](#contributing)\n\n<!-- tocstop -->\n\n## Requirements\n\n* Python 3.6+\n\n\n## Install\n\n```python\npip install pyartifactory\n```\n\n## Usage\n\n### Authentication\n\nSince Artifactory 6.6.0 there is version 2 of the REST API for permission management, in case you have that version or higher, you need to pass api_version=2 to the constructor when you instantiate the class.\n\n```python\nfrom pyartifactory import Artifactory\nart = Artifactory(url="ARTIFACTORY_URL", auth=(\'USERNAME\',\'PASSWORD_OR_API_KEY\'), api_version=1)\n```\n\n### SSL Cert Verification Options\nSpecify a local cert to use as client side certificate\n\n```python\nfrom pyartifactory import Artifactory\nart = Artifactory(url="ARTIFACTORY_URL", auth=(\'USERNAME\',\'PASSORD_OR_API_KEY\'), cert="/path_to_file/server.pem",api_version=1)\n```\n\nDisable host cert verification\n\n```python\nfrom pyartifactory import Artifactory\nart = Artifactory(url="ARTIFACTORY_URL", auth=(\'USERNAME\',\'PASSORD_OR_API_KEY\'), verify=False, api_version=1)\n```\n\n### Admin objects\n\n#### User\n\nFirst, you need to create a new Artifactory object.\n```python\nfrom pyartifactory import Artifactory\nart = Artifactory(url="ARTIFACTORY_URL", auth=(\'USERNAME\',\'PASSORD_OR_API_KEY\'))\n```\n\nGet the list of users:\n```python\nusers = art.users.list()\n```\n\nGet a single user:\n```python\nuser = art.users.get("test_user")\n```\n\nCreate a user:\n```python\nfrom pyartifactory.models import NewUser\n\n# Create User\nuser = NewUser(name="test_user", password="test_password", email="user@user.com")\nnew_user = art.users.create(user)\n\n# Update user\nuser.email = "test@test.com"\nupdated_user = art.users.update(user)\n```\n\nUpdate a user:\n```python\nfrom pyartifactory.models import User\n\nuser = art.users.get("test_user")\n\n# Update user\nuser.email = "test@test.com"\nupdated_user = art.users.update(user)\n```\n\nDelete a user:\n```python\nart.users.delete("test_user")\n```\n\nUnlock a user:\n```python\nart.users.unlock("test_user")\n```\n\n#### Group\n\nGet the list of groups:\n```python\ngroups = art.groups.list()\n```\n\nGet a single group:\n```python\ngroup = art.groups.get("group_name")\n```\n\nCreate/Update a group:\n```python\nfrom pyartifactory.models import Group\n\n# Create a Group\ngroup = Group(name="test_group", description="test_group")\nnew_group = art.groups.create(group)\n\n# Update a Group\ngroup.description = "test_group_2"\nupdated_group = art.groups.update(group)\n```\n\nDelete a group:\n```python\nart.groups.delete("test_group")\n```\n\n#### Security\n\nA set of methods for performing operations on apiKeys, passwords ...\n```python\n>>> art.security.\nart.security.create_api_key(          art.security.get_encrypted_password(  art.security.revoke_api_key(\nart.security.get_api_key(             art.security.regenerate_api_key(      art.security.revoke_user_api_key(\n```\n\nCreate an access token (for a transient user):\n```python\ntoken = art.security.create_access_token(user_name=\'transient_artifactory_user\',\n                                         groups=[\'g1\', \'g2\'],\n                                         refreshable=True)\n```\n\nCreate an access token for an existing user (groups are implied from the existing user):\n```python\ntoken = art.security.create_access_token(user_name=\'existing_artifactory_user\',\n                                         refreshable=True)\n```\n\nRevoke an existing revocable token:\n```python\nart.security.revoke_access_token(token.access_token)\n```\n\n#### Repository\n\nGet the list of repositories:\n```python\nrepositories = art.repositories.list()\n```\n\nGet a single repository\n```python\nrepo = art.repositories.get_repo("repo_name")\n# According to the repo type, you\'ll have either a local, virtual or remote repository returned\n```\n\nCreate/Update a repository:\n```python\nfrom pyartifactory.models import LocalRepository, VirtualRepository, RemoteRepository\n\n# Create local repo\nlocal_repo = LocalRepository(key="test_local_repo")\nnew_local_repo = art.repositories.create_repo(local_repo)\n\n# Create virtual repo\nvirtual_repo = VirtualRepository(key="test_virtual_repo")\nnew_virtual_repo = art.repositories.create_repo(virtual_repo)\n\n# Create remote repo\nremote_repo = RemoteRepository(key="test_remote_repo")\nnew_remote_repo = art.repositories.create_repo(remote_repo)\n\n# Update a repository\nlocal_repo = art.repositories.get_repo("test_local_repo")\nlocal_repo.description = "test_local_repo"\nupdated_local_repo = art.repositories.update_repo(local_repo)\n```\n\nDelete a repository:\n```python\nart.repositories.delete("test_local_repo")\n```\n\n#### Permission\nGet the list of permissions:\n```python\npermissions = art.permissions.list()\n```\n\nGet a single permission:\n```python\nusers = art.permissions.get("test_permission")\n```\n\nCreate/Update a permission:\n\n##### Artifactory lower than 6.6.0\n\n```python\n\nfrom pyartifactory.models import Permission\n\n# Create a permission\npermission = Permission(\n    **{\n        "name": "test_permission",\n        "repositories": ["test_repository"],\n        "principals": {\n            "users": {"test_user": ["r", "w", "n", "d"]},\n            "groups": {"developers": ["r"]},\n        },\n    }\n)\nperm = art.permissions.create(permission)\n\n# Update permission\npermission.repositories = ["test_repository_2"]\nupdated_permission = art.permissions.update(permission)\n```\n\n##### Artifactory 6.6.0 or higher\n```python\nfrom pyartifactory import Artifactory\nfrom pyartifactory.models import PermissionV2\nfrom pyartifactory.models.permission import PermissionEnumV2, PrincipalsPermissionV2, RepoV2, BuildV2, ReleaseBundleV2\n\n# To use PermissionV2, make sure to set api_version=2\nart = Artifactory(url="ARTIFACTORY_URL", auth=(\'USERNAME\',\'PASSWORD_OR_API_KEY\'), api_version=2)\n\n# Create a permission\npermission = PermissionV2(\n    name="test_permission",\n    repo=RepoV2(\n        repositories=["test_repository"],\n        actions=PrincipalsPermissionV2(\n            users={\n                "test_user": [\n                    PermissionEnumV2.read,\n                    PermissionEnumV2.annotate,\n                    PermissionEnumV2.write,\n                    PermissionEnumV2.delete,\n                ]\n            },\n            groups={\n                "developers": [\n                    PermissionEnumV2.read,\n                    PermissionEnumV2.annotate,\n                    PermissionEnumV2.write,\n                    PermissionEnumV2.delete,\n                ],\n            },\n        ),\n        includePatterns=["**"],\n        excludePatterns=[],\n    ),\n    build=BuildV2(\n          actions=PrincipalsPermissionV2(\n              users={\n                  "test_user": [\n                      PermissionEnumV2.read,\n                      PermissionEnumV2.write,\n                  ]\n              },\n              groups={\n                  "developers": [\n                      PermissionEnumV2.read,\n                      PermissionEnumV2.write,\n                  ],\n              },\n          ),\n          includePatterns=[""],\n          excludePatterns=[""],\n      ),\n    releaseBundle=ReleaseBundleV2(\n          repositories=["release-bundles"],\n          actions=PrincipalsPermissionV2(\n              users={\n                  "test_user": [\n                      PermissionEnumV2.read,\n                  ]\n              },\n              groups={\n                  "developers": [\n                      PermissionEnumV2.read,\n                  ],\n              },\n          ),\n          includePatterns=[""],\n          excludePatterns=[""],\n      )\n  # You don\'t have to set all the objects repo, build and releaseBundle\n  # If you only need repo for example, you can set only the repo object\n)\nperm = art.permissions.create(permission)\n\n# Update permission\npermission.repo.repositories = ["test_repository_2"]\nupdated_permission = art.permissions.update(permission)\n```\n\nDelete a permission:\n```python\nart.permissions.delete("test_permission")\n```\n\n### Artifacts\n\n#### Get the information about a file or folder\n```python\nartifact_info = art.artifacts.info("<ARTIFACT_PATH_IN_ARTIFACTORY>")\n# file_info = art.artifacts.info("my-repository/my/artifact/directory/file.txt")\n# folder_info = art.artifacts.info("my-repository/my/artifact/directory")\n```\n\n#### Deploy an artifact\n```python\nartifact = art.artifacts.deploy("<LOCAL_FILE_LOCATION>", "<ARTIFACT_PATH_IN_ARTIFACTORY>")\n# artifact = art.artifacts.deploy("Desktop/myNewFile.txt", "my-repository/my/new/artifact/directory/file.txt")\n```\n\n#### Download an artifact\n```python\nartifact = art.artifacts.download("<ARTIFACT_PATH_IN_ARTIFACTORY>", "<LOCAL_DIRECTORY_PATH>")\n# artifact = art.artifacts.download("my-artifactory-repository/my/new/artifact/file.txt", "Desktop/my/local/directory")\n# The artifact location is returned by the download method\n# If you have not set a <LOCAL_DIRECTORY_PATH>, the artifact will be downloaded in the current directory\n```\n\n#### Retrieve artifact list\n```python\nartifacts = art.artifacts.list("<ARTIFACT_PATH_IN_ARTIFACTORY>")\n# files_only = art.artifacts.list("<ARTIFACT_PATH_IN_ARTIFACTORY>", list_folders=False)\n# non_recursive = art.artifacts.list("<ARTIFACT_PATH_IN_ARTIFACTORY>", recursive=False)\n# max_depth = art.artifacts.list("<ARTIFACT_PATH_IN_ARTIFACTORY>", depth=3)\n```\n\n#### Retrieve artifact properties\n```python\nartifact_properties = art.artifacts.properties("<ARTIFACT_PATH_IN_ARTIFACTORY>")  # returns all properties\n# artifact_properties = art.artifacts.properties("my-repository/my/new/artifact/directory/file.txt")\nartifact_properties = art.artifacts.properties("<ARTIFACT_PATH_IN_ARTIFACTORY>", ["prop1", "prop2"])  # returns specific properties\nartifact_properties.properties["prop1"]  # ["value1", "value1-bis"]\n```\n\n#### Set artifact properties\n```python\nartifact_properties = art.artifacts.set_properties("<ARTIFACT_PATH_IN_ARTIFACTORY>", {"prop1": "value", "prop2": "value"})  # recursive mode is enabled by default\nartifact_properties = art.artifacts.set_properties("<ARTIFACT_PATH_IN_ARTIFACTORY>", {"prop1": "value", "prop2": "value"}, False) # disable recursive mode\n```\n\n#### Update artifact properties\n```python\nartifact_properties = art.artifacts.update_properties("<ARTIFACT_PATH_IN_ARTIFACTORY>", {"prop1": "value", "prop2": "value"})  # recursive mode is enabled by default\nartifact_properties = art.artifacts.update_properties("<ARTIFACT_PATH_IN_ARTIFACTORY>", {"prop1": "value", "prop2": "value"}, False) # disable recursive mode\n```\n\n#### Retrieve artifact stats\n```python\nartifact_stats = art.artifacts.stats("<ARTIFACT_PATH_IN_ARTIFACTORY>")\n# artifact_stats = art.artifacts.stats("my-repository/my/new/artifact/directory/file.txt")\n```\n\n#### Copy artifact to a new location\n```python\nartifact = art.artifacts.copy("<CURRENT_ARTIFACT_PATH_IN_ARTIFACTORY>","<NEW_ARTIFACT_PATH_IN_ARTIFACTORY>")\n\n# If you want to run a dryRun test, you can do the following:\n# artifact = art.artifacts.copy("my-repository/current/artifact/path/file.txt","my-repository/new/artifact/path/file.txt", dryrun=True)\n# It will return properties of the newly copied artifact\n```\n\n#### Move artifact to a new location\n```python\nartifact = art.artifacts.move("<CURRENT_ARTIFACT_PATH_IN_ARTIFACTORY>","<NEW_ARTIFACT_PATH_IN_ARTIFACTORY>")\n\n# You can also run a dryRun test with the move operation\n# It will return properties of the newly moved artifact\n```\n\n#### Delete an artifact\n```python\nart.artifacts.delete("<ARTIFACT_PATH_IN_ARTIFACTORY>")\n```\n\n\n### Contributing\nPlease read the [Development - Contributing](./CONTRIBUTING.md) guidelines.\n',
    'author': 'Ananias CARVALHO',
    'author_email': 'carvalhoananias@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/anancarv/python-artifactory',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
